package edu.upvehu.gbg.babel;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Map;
import java.util.TreeMap;

public class Babel {
    Map<String,Language> lenguajes=new TreeMap<>();
    
    Babel() throws IOException{
        try (BufferedReader br=new BufferedReader(new FileReader("udhr/languages.txt",Charset.forName("latin1")))){ 
            for( String linea; (linea=br.readLine())!=null; ) {
                String[] campos=linea.split(" ",2);
                lenguajes.put(campos[0],new Language(campos[0],campos[1],"udhr/txt/"+campos[0]+".txt"));}}}

    public Language[] findMostSimilars(String langId, int n){
        if (n<=0 || n>=lenguajes.size())
            throw new IllegalArgumentException("El número de lengajes solicitado ("+n+") excede los disponibles.");
        Language refLang= langId==null?null:lenguajes.get(langId);
        if (refLang==null)
            throw new IllegalArgumentException("Identificador de lenguaje ("+langId+") desconocido.");

        class LanguageAndSimilarity implements Comparable<LanguageAndSimilarity>{
            private final Language LANGUAGE; private final double SIMILARITY;
            public LanguageAndSimilarity(Language lang, double dist) {
                LANGUAGE = lang; SIMILARITY = dist;}
            @Override public int compareTo(LanguageAndSimilarity other) {
                return  other == null? 1: this==other?0:
                       Double.compare(other.SIMILARITY, SIMILARITY);}}

        LanguageAndSimilarity[] LangAndSimilarityArray=new LanguageAndSimilarity[lenguajes.size()-1];
        {int i=0; for(String lid:lenguajes.keySet()) if (!lid.equals(langId)) {
            Language checkLang=lenguajes.get(lid);
            LangAndSimilarityArray[i++]=new LanguageAndSimilarity(checkLang,Language.similarity(refLang, checkLang));}}
        Arrays.sort(LangAndSimilarityArray);
        Language[] result=new Language[n];
        for (int i=0; i<result.length;i++) result[i]=LangAndSimilarityArray[i].LANGUAGE;
        return result;}
    
    public Language[] findMostSimilars2(String langId, int n){
        if (n<=0 || n>=lenguajes.size())
            throw new IllegalArgumentException("El número de lengajes solicitado ("+n+") excede los disponibles.");
        Language refLang= langId==null?null:lenguajes.get(langId);
        if (refLang==null)
            throw new IllegalArgumentException("Identificador de lenguaje ("+langId+") desconocido.");

        class LanguageAndSimilarity{
            private final Language LANGUAGE; private final double SIMILARITY;
            public LanguageAndSimilarity(Language lang, double dist) {
                LANGUAGE = lang; SIMILARITY = dist;}
            }

        LanguageAndSimilarity[] LangAndSimilarityArray=new LanguageAndSimilarity[lenguajes.size()-1];
        {int i=0; for(String lid:lenguajes.keySet()) if (!lid.equals(langId)) {
            Language checkLang=lenguajes.get(lid);
            LangAndSimilarityArray[i++]=new LanguageAndSimilarity(checkLang,Language.similarity(refLang, checkLang));}}
        Arrays.sort(LangAndSimilarityArray, new Comparator<LanguageAndSimilarity>(){
            @Override public int compare(LanguageAndSimilarity ls1, LanguageAndSimilarity ls2) {
                return  ls1==ls2?0: ls1==null? 1: ls2==null? -1:
                       Double.compare(ls2.SIMILARITY, ls1.SIMILARITY);}
        });
        Language[] result=new Language[n];
        for (int i=0; i<result.length;i++) result[i]=LangAndSimilarityArray[i].LANGUAGE;
        return result;}
    
    public Language[] findMostSimilars3(String langId, int n){
        if (n<=0 || n>=lenguajes.size())
            throw new IllegalArgumentException("El número de lengajes solicitado ("+n+") excede los disponibles.");
        Language refLang= langId==null?null:lenguajes.get(langId);
        if (refLang==null)
            throw new IllegalArgumentException("Identificador de lenguaje ("+langId+") desconocido.");

        record LanguageAndSimilarity (Language language, double similarity) implements Comparable<LanguageAndSimilarity>{
            @Override public int compareTo(LanguageAndSimilarity other) {
                return  other == null? 1: this==other?0:
                       Double.compare(other.similarity, similarity);}}

        LanguageAndSimilarity[] LangAndSimilarityArray=new LanguageAndSimilarity[lenguajes.size()-1];
        {int i=0; for(String lid:lenguajes.keySet()) if (!lid.equals(langId)) {
            Language checkLang=lenguajes.get(lid);
            LangAndSimilarityArray[i++]=new LanguageAndSimilarity(checkLang,Language.similarity(refLang, checkLang));}}
        Arrays.sort(LangAndSimilarityArray);
        Language[] result=new Language[n];
        for (int i=0; i<result.length;i++) result[i]=LangAndSimilarityArray[i].language;
        return result;}  
    
        public Language[] findMostSimilars4(String langId, int n){
        if (n<=0 || n>=lenguajes.size())
            throw new IllegalArgumentException("El número de lengajes solicitado ("+n+") excede los disponibles.");
        Language refLang= langId==null?null:lenguajes.get(langId);
        if (refLang==null)
            throw new IllegalArgumentException("Identificador de lenguaje ("+langId+") desconocido.");

        Language[] LangArray=lenguajes.values().toArray(new Language[0]);
        Arrays.sort(LangArray, new Comparator<Language>(){
            @Override public int compare(Language l1, Language l2) {
                return  l1==l2?0: l1==null? 1: l2==null? -1:
                       Double.compare(Language.similarity(refLang, l2), Language.similarity(refLang, l1));}
        });
        return Arrays.copyOfRange(LangArray,1, n);}
    
        public Language[] findMostSimilars5(String langId, int n){
        if (n>=lenguajes.size())
            throw new IllegalArgumentException("El número de lengajes solicitado ("+n+") excede los disponibles.");
        Language refLang= langId==null?null:lenguajes.get(langId);
        if (refLang==null)
            throw new IllegalArgumentException("Identificador de lenguaje ("+langId+") desconocido.");

        Language[] LangArray=lenguajes.values().toArray(new Language[0]);
        Arrays.sort(LangArray, (Language l1, Language l2) -> l1==l2?0: l1==null? 1: l2==null? -1:
                Double.compare(Language.similarity(refLang, l2), Language.similarity(refLang, l1)));
        return Arrays.copyOfRange(LangArray,1, n);}

    Language getLanguage(String langId) {
        return lenguajes.get(langId);
    }
        
        
        
}
